﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Observable
{
	/// <summary>
	/// 数値計算用のドメインクラスです。
	/// </summary>
	public class Interval : ISubject
	{
		#region フィールド
		/// <summary>
		/// オブザーバーのリスト
		/// </summary>
		private List<IObserver> _observers;

		/// <summary>
		/// Start値
		/// </summary>
		private string _start = "0";
		/// <summary>
		/// End値
		/// </summary>
		private string _end = "0";
		/// <summary>
		/// Length値
		/// </summary>
		private string _length = "0";
		#endregion

		#region プロパティ
		/// <summary>
		/// ドメインクラスのStart値の取得と設定をします。変更はオブザーバーに通知されます。
		/// </summary>
		public string Start
		{
			get
			{
				return this._start;
			}
			set
			{
				this._start = value;
				SetChanged();
				NotifyObservers();
			}
		}

		/// <summary>
		/// ドメインクラスのEnd値の取得と設定をします。変更はオブザーバーに通知されます。
		/// </summary>
		public string End
		{
			get
			{
				return this._end;
			}
			set
			{
				this._end = value;
				SetChanged();
				NotifyObservers();
			}
		}

		/// <summary>
		///ドメインクラスの Length値の取得と設定をします。変更はオブザーバーに通知されます。
		/// </summary>
		public string Length
		{
			get
			{
				return this._length;
			}
			set
			{
				this._length = value;
				SetChanged();
				NotifyObservers();
			}
		}
		#endregion

		#region コンストラクタ・生成
		/// <summary>
		/// 数値計算用のドメインクラスです。
		/// </summary>
		public Interval()
		{
			this._observers = new List<IObserver>();
		}
		#endregion

		#region メソッド
		/// <summary>
		/// 
		/// </summary>
		private void SetChanged()
		{
		}

		/// <summary>
		/// オブザーバーをドメインクラスへ登録し、変更の通知を受け取れるようにします。
		/// </summary>
		/// <param name="observer">サブジェクトに登録するオブザーバーのインスタンス</param>
		public void RegisterInterest(IObserver observer)
		{
			this._observers.Add( observer );
		}

		/// <summary>
		/// 指定したオブジェクトがオブザーバーとして登録済みかどうかを判定します。
		/// </summary>
		/// <param name="observer">判定したいオブジェクト</param>
		/// <returns>登録済みならTrue</returns>
		public bool IsInterest( IObserver observer )
		{
			return this._observers.Contains( observer );
		}

		/// <summary>
		/// 変更をオブザーバーに通知します。
		/// </summary>
		protected virtual void NotifyObservers()
		{
			_observers.ForEach( observer => observer.NotifyUpdate( this ) );
		}

		/// <summary>
		/// Lengthを再計算して、更新します。
		/// </summary>
		public virtual void CalculateLength()
		{
			int start = ForceInt( this.Start );
			int end = ForceInt( this.End );
			int length = end - start;

			this.Length = length.ToString();
		}

		/// <summary>
		/// Endを再計算して、更新します。
		/// </summary>
		public virtual void CalculateEnd()
		{
			int start = ForceInt( this.Start );
			int length = ForceInt( this.Length );
			int end = start + length;

			this.End = end.ToString();
		}

		/// <summary>
		/// 文字列を整数に変換します。失敗するとゼロを返します。
		/// </summary>
		/// <param name="input">変換したい文字列</param>
		/// <returns>変換後の整数。</returns>
		protected virtual int ForceInt( string input )
		{
			if( !isNumber( input ) )
			{
				return 0;
			}
			return int.Parse( input );
		}

		/// <summary>
		/// 入力値が数値かどうかを検証する。
		/// </summary>
		/// <param name="input">入力値</param>
		/// <returns>数値ならTrue</returns>
		protected virtual bool isNumber( string input )
		{
			int result = 0;
			return int.TryParse( input, out result );
		}
		#endregion
	}
}
